#include <string>
#include <iostream>
#include <FreeImage.h>
#include <cstdlib>

using namespace std;

#define WIDTH 800
#define HEIGHT 800
#define BPP 24
#define MAX_ITER 5000

void getPixelColor(int i, int j, RGBQUAD & color, double llx, double lly, double s) {
	
	double di = (double)i/(double)WIDTH;
	double dj = (double)j/(double)HEIGHT;
	double cx = llx + s*di;
	double cy = lly + s*dj;
		
	double x = 0;
	double y = 0;
	int iter;
	for (iter = 0; iter < MAX_ITER && (x*x+y*y) < 4; iter++) {
		
		double newx = x*x - y*y + cx;
		double newy = 2*x*y + cy;
		
		x = newx;
		y = newy;
		
	}
	if (iter == MAX_ITER) {

		color.rgbRed = 0;
		color.rgbGreen = 0;
		color.rgbBlue = 0;
		
	} else {
		
		//double d = ((double)iter/MAX_ITER);
		
		color.rgbRed = 255;
		color.rgbGreen = 255;
		color.rgbBlue = 255;
	
	}
		
}
/**
 * Generates an image of the complex plane from lower left corner x + iy with sidelengths s.
 */
int generateImage(double llx, double lly, double s, char* filename) {
	FreeImage_Initialise();
	
	FIBITMAP* bitmap = FreeImage_Allocate(WIDTH, HEIGHT, BPP);
	RGBQUAD color;
	
	if (!bitmap) {
		cerr << "Could not allocate bitmap!" << endl;
		return 1;
	}
	
	for (int x = 0; x < WIDTH; x++) {
		for (int y = 0; y < HEIGHT; y++) {
			getPixelColor(x,y,color,llx,lly,s);
			FreeImage_SetPixelColor(bitmap, x, y, &color);
		}
	}
	
	if (FreeImage_Save(FIF_PNG, bitmap, filename, 0)) {
		cout << "Image successfully saved!" << endl;
	}
	
	FreeImage_DeInitialise();
	return 0;
}

int main(int argc, char* argv[]) {
	
	cout << "Mandelbrot generator loaded." << endl;
	
	if (argc != 5) {
		cerr << "Expected 4 arguments: x y s output.png" << endl;
		exit(1);
	}
	
	double llx = atof(argv[1]);
	double lly = atof(argv[2]);
	double s = atof(argv[3]);
	char* filename = argv[4];
	
	int success = generateImage(llx, lly, s, filename);
	return success;
}